function varargout=dd1(flag,varargin)
% *********************************************************************
% Implements a (DD1) Kalman filter.
% ---------------------------------------------------------------------
% Usage: [xht,St,yht,rest,Rt]=dd1(3,pred,yt,ut) % off line version
%                             dd1(3,pred,yt)    % off line version
%        dd1(0,pred)               % Initialisering recursive version
%        [xh,P,res,R,K]=dd1(1,y)   % Data update
%        [xh,P]=dd1(2,u)           % Time update
%        [xh,P]=dd1(2)             % Time update
% -------------------------------------------------------------------------
% yt    data sequence of measurements (outputs)
% ut    data sequence of inputs
% y	    measurement
% u	    input
% pred  predictive (1) or ordinary (0) version
% *************************************************************************

persistent pred h
persistent nx nu nv ny ne
persistent x0 P0 vm R1 em R2
persistent xh P Sv Se

switch flag,
 % *********************************************************************
 case 0, % Initialisering
  if nargin>1, pred=varargin{1}; else, pred=0; end;
  [stype,nx,nu,nv,ny,ne]=initt(3);
  [x0,P0,vm,R1,em,R2]=initt(4);
  Sv=msqrt(R1); Se=msqrt(R2);
  xh=x0; P=P0;
  h=sqrt(3);
  varargout={xh, P};
 % ------------------------------------------------------------------------
 % Data update
 % ------------------------------------------------------------------------
 case 1,
  y=varargin{1};
  Sx=msqrt(P);
  xa=kron(ones(1,nx),xh); xb=kron(ones(1,ne),xh);
  ea=kron(ones(1,ne),em); eb=kron(ones(1,nx),em);
  X=[xh xa+h*Sx xb xa-h*Sx xb];
  E=[em eb   ea+h*Se eb ea-h*Se];
  Y=measmodel([X; E]);
    
  yh=Y(:,1); Y(:,1)=[]; Dyp=Y(:,1:(nx+ne)); Y(:,1:(nx+ne))=[]; Dyn=Y;

  Dy=Dyp-Dyn;
  R=Dy*Dy'/4/h^2;
  Pxy=Sx*Dy(:,1:nx)'/2/h;

  K=Pxy*inv(R); 

  res=y-yh;
  if ~misobs(y),
   xh=xh+K*res;
   P=P-K*Pxy';
  end;

  varargout={xh,P,res,yh,R,K};
 % -----------------------------------------------------------------------
 % Time update
 % ------------------------------------------------------------------------
 case 2
  if nu>0, u=varargin{1}; else, u=[]; end;
  Sx=msqrt(P);
  xa=kron(ones(1,nx),xh); xb=kron(ones(1,nv),xh);
  va=kron(ones(1,nv),vm); vb=kron(ones(1,nx),vm);
  X=[xh xa+h*Sx xb xa-h*Sx xb];
  if nu>0, U=kron(ones(1,1+2*(nx+nv)),u); else, U=[]; end;
  V=[vm vb va+h*Sv vb va-h*Sv];
  X=statemodel([X; U; V]);

  xh=X(:,1);  X(:,1)=[]; Dxp=X(:,1:(nx+nv)); X(:,1:(nx+nv))=[]; Dxn=X;
  Dx=Dxp-Dxn;
  P=Dx*Dx'/4/h^2;
  varargout={xh,P};
 % *********************************************************************
 otherwise
  % *********************************************************************
end % flag end

