function varargout=klms(flag,varargin)
% *********************************************************************
% Implements a (Stationary) Kalman filter.
% ---------------------------------------------------------------------
% Usage: [xht,St,yht,rest,Rt]=klms(3,pred,yt,ut) % off line version
%                             klms(3,pred,yt)    % off line versio
%        [xh,P]=klms(0,pred)     % Initialisering recursive version
%        [xh,res]=klms(1,y)      % Data update
%        [xh]=klms(2,u)          % Time update
%        [xh]=klms(2)            % Time update
% ---------------------------------------------------------------------
% Notice point of linearization can be introduced in statelin and measlin
% ---------------------------------------------------------------------
% yt    measurements (outputs)
% ut    inputs
% y         measurement
% u         input
% pred  predictive (1) or ordinary (0) version
% *********************************************************************
 persistent pred
 persistent nx nu nv ny ne
 persistent x0 P0 vm R1 em R2
 persistent A B C Gx Gy
 persistent xh yh P Q R K

switch flag,
% *********************************************************************
case 0, % Initialize
 if nargin>1, pred=varargin{1}; else, pred=0; end;
 [stype,nx,nu,nv,ny,ne]=initt(3);
 [x0,P0,vm,R1,em,R2]=initt(4);
 xh=x0; P=P0;
 [A,B,Gx]=statelin; % LTI system
 if ~isempty(Gx), vm=Gx*vm; R1=Gx*R1*Gx'; end;
 Gx=eye(nx); 
 [C,D,Gy]=measlin;
 if ~isempty(Gy), em=Gy*em; R2=Gy*R2*Gy'; Gy=eye(ny); end;
 [K,P,Q]=dlqe(A,Gx,C,R1,R2); % P predictive, Q ordinare
 if ~pred, P=Q; end;
 R=C*P*C'+R2;
 varargout={xh, P};
 % ------------------------------------------------------------------------
 % Data update
 % ------------------------------------------------------------------------
 case 1,
  y=varargin{1};
  yh=C*xh+em;
  res=y-yh;
  if ~misobs(y),
   xh=xh+K*res;
  end;
  varargout={xh,P,res,yh,R,K};
 % -----------------------------------------------------------------------
 % Time update
 % ------------------------------------------------------------------------
 case 2
  if nu>0, u=varargin{1}; xh=A*xh+B*u+vm; else, xh=A*xh+vm; end;
  varargout={xh,P};
  % *********************************************************************
 otherwise
  % *********************************************************************
end % flag end
