function varargout=lrkf(flag,varargin)
% *********************************************************************
% Implements a (Linear Regression) Kalman filter.
% ---------------------------------------------------------------------
% Usage: [xht,St,yht,rest,Rt]=lrkf(3,pred,yt,ut) % off line version
%                             lrkf(3,pred,yt)    % off line version
%        lrkf(0,pred)               % Initialisering recursive version
%        [xh,P,res,R,K]=lrkf(1,y)   % Data update
%        [xh,P]=lrkf(2,u)           % Time update
%        [xh,P]=lrkf(2)             % Time update
% -------------------------------------------------------------------------
% yt    data sequence of measurements (outputs)
% ut    data sequence of inputs
% y	    measurement
% u	    input
% pred  predictive (1) or ordinary (0) version
% *************************************************************************

persistent pred h
persistent nx nu nv ny ne
persistent x0 P0 vm R1 em R2
persistent xh P Sv Se

switch flag,
 % *********************************************************************
 case 0, % Initialisering
  if nargin>1, pred=varargin{1}; else, pred=0; end;
  [stype,nx,nu,nv,ny,ne]=initt(3);
  [x0,P0,vm,R1,em,R2]=initt(4);
  Sv=msqrt(R1); Se=msqrt(R2);
  xh=x0; P=P0;
  h=sqrt(3);
  varargout={xh, P};
  % ------------------------------------------------------------------------
  % Data update
  % ------------------------------------------------------------------------
 case 1,
  y=varargin{1};
  S2=msqrt(catdiag(P,R2));
  x2=[xh; em];
  X=[x2 kron(ones(1,nx+ne),x2)+h*S2 kron(ones(1,nx+ne),x2)-h*S2];
  Xr=[X; ones(1,1+2*(nx+ne))];
  Y=measmodel(X);

  yh=mean(Y')';
  th=Y*Xr'*inv(Xr*Xr');
  C=th(:,1:nx);
  Gy=th(:,nx+1:end-1);
  % b=th(:,end);

  R=C*P*C'+Gy*R2*Gy';
  K=P*C'*inv(R);

  res=y-yh;
  if ~misobs(y),
   xh=xh+K*res;
   P=P-K*C*P';
  end;

  varargout={xh,P,res,yh,R,K};
  % -----------------------------------------------------------------------
  % Time update
  % ------------------------------------------------------------------------
 case 2
  if nu>0, u=varargin{1}; else, u=[]; end;
  S1=msqrt(catdiag(P,R1));
  x1=[xh; vm];
  X=[x1 kron(ones(1,nx+nv),x1)+h*S1 kron(ones(1,nx+nv),x1)-h*S1];
  if nu>0,
   U=kron(ones(1,1+2*(nx+nv)),u);
   Xi=[X(1:nx,:); U; X(nx+1:end,:)];
  else
   Xi=X;
  end
  Xr=[X; ones(1,1+2*(nx+nv))];

  X=statemodel(Xi);
  xh=mean(X')';

  Xy=X;
  th=Xy*Xr'*pinv(Xr*Xr');
  A=th(:,1:nx);
  G=th(:,nx+1:end-1);
  % b=th(:,end);
  P=A*P*A'+G*R1*G';

  varargout={xh,P};
  % *********************************************************************
 otherwise
  % *********************************************************************
end % flag end

