function varargout=offline(ftype,varargin)
% *************************************************************************
% Usage: [xht,St,yht,rest,Rt]=offline(ftype,pred,yt,ut);
% 
% where
% 
% ftype: filter type 0-8 (see filts below)
% pred:  preditive (1) version or ordinary (otherwise)
% yt:    sequence of measurements
% ut:    sequence of inputs (if exists).
% *************************************************************************
% xht   state estimate (as function of time coloumn wise)
% rest  residual or prediction error
% yht   predicted output
% St    Std of state estimation error
% Rt    Std of prediction error
% -------------------------------------------------------------------------

pred=varargin{1};
[xh,P]=filts(ftype,0,pred);
[stype,nx,nu,nv,ny,ne]=initt(3);
Yt=varargin{2};
if nu>0, Ut=varargin{3}; end;

xht=[]; St=[]; yht=[]; rest=[]; Rt=[];

Ns=length(Yt);
for i=1:Ns,
 fprintf(' %4i \r',i)
 if pred,
  xht=[xht; xh'];
  St=[St; sqrt(diag(P))'];
 end;

 y=Yt(:,i); % Data update
 [xh,P,res,yh,R,K]=filts(ftype,1,y);
 rest=[rest; res'];
 yht=[yht; yh'];
 Rt=[Rt; sqrt(diag(R))'];

 if ~pred,
  xht=[xht; xh'];
  St=[St; sqrt(diag(P))'];
 end;
 if nu>0, % Time update
  u=Ut(:,i);
  [xh,P]=filts(ftype,2,u);
 else,
  [xh,P]=filts(ftype,2);
 end;
end;
xht=xht'; St=St'; yht=yht'; rest=rest'; Rt=Rt';
varargout={xht; St; yht; rest; Rt};

function varargout=filts(ftype,call,varargin)

switch ftype,
 case 'klms',
  switch call,
   case 0, [xh,P]=klms(0,varargin{:});varargout={xh,P};
   case 1, [xh,P,res,yh,R,K]=klms(1,varargin{:}); varargout={xh,P,res,yh,R,K};
   case 2, [xh,P]=klms(2,varargin{:}); varargout={xh,P};
  end;
 case 'klm',
  switch call,
   case 0, [xh,P]=klm(0,varargin{:});varargout={xh,P};
   case 1, [xh,P,res,yh,R,K]=klm(1,varargin{:}); varargout={xh,P,res,yh,R,K};
   case 2, [xh,P]=klm(2,varargin{:}); varargout={xh,P};
  end;
 case 'ekf',
  switch call,
   case 0, [xh,P]=ekf(0,varargin{:});varargout={xh,P};
   case 1, [xh,P,res,yh,R,K]=ekf(1,varargin{:}); varargout={xh,P,res,yh,R,K};
   case 2, [xh,P]=ekf(2,varargin{:}); varargout={xh,P};
  end;
 case 'ukf',
  switch call,
   case 0, [xh,P]=ukf(0,varargin{:});varargout={xh,P};
   case 1, [xh,P,res,yh,R,K]=ukf(1,varargin{:}); varargout={xh,P,res,yh,R,K};
   case 2, [xh,P]=ukf(2,varargin{:}); varargout={xh,P};
  end;
 case 'dd1',
  switch call,
   case 0, [xh,P]=dd1(0,varargin{:});varargout={xh,P};
   case 1, [xh,P,res,yh,R,K]=dd1(1,varargin{:}); varargout={xh,P,res,yh,R,K};
   case 2, [xh,P]=dd1(2,varargin{:}); varargout={xh,P};
  end;
 case 'dd2',
  switch call,
   case 0, [xh,P]=dd2(0,varargin{:});varargout={xh,P};
   case 1, [xh,P,res,yh,R,K]=dd2(1,varargin{:}); varargout={xh,P,res,yh,R,K};
   case 2, [xh,P]=dd2(2,varargin{:}); varargout={xh,P};
  end;
 case 'lrkf',
  switch call,
   case 0, [xh,P]=lrkf(0,varargin{:});varargout={xh,P};
   case 1, [xh,P,res,yh,R,K]=lrkf(1,varargin{:}); varargout={xh,P,res,yh,R,K};
   case 2, [xh,P]=lrkf(2,varargin{:}); varargout={xh,P};
  end;
 case 'skfp',
  switch call,
   case 0, [xh,P]=skfp(0,varargin{:});varargout={xh,P};
   case 1, [xh,P,res,yh,R,K]=skfp(1,varargin{:}); varargout={xh,P,res,yh,R,K};
   case 2, [xh,P]=skfp(2,varargin{:}); varargout={xh,P};
  end;
   case 'skfb',
  switch call,
   case 0, [xh,P]=skfb(0,varargin{:});varargout={xh,P};
   case 1, [xh,P,res,yh,R,K]=skfb(1,varargin{:}); varargout={xh,P,res,yh,R,K};
   case 2, [xh,P]=skfb(2,varargin{:}); varargout={xh,P};
  end;

 otherwise, fprintf('ftype out of range in offline/filts \n'); pause
end
